==> small-json-diff-1.1.0.tgz: package/README.md <==
# small-json-diff

Library for creating small [RFC6902](http://tools.ietf.org/html/rfc6902) compliant json diffs. The char length of the diff is guaranteed to be smaller than that of the diff produced by replacing the entire object. This is recursively true for each sub-object.

The algorithm will only produce "remove" and "replace" operations. The diff produced will be close to the smallest possible as long as no arrays are used. The reason is that it's hard to detect when items are moved (gets a new index position) inside an array. When that happens the item will be compared to the item that was at the new position before the move. But at least the diff size required to modify an array will never be larger than the diff size required to replace the entire array with the new array.

## Install

Install the current version (and save it as a dependency):

```sh
$ npm install small-json-patch --save
```

## Usage

```js
var SmallJsonPatch = require("small-json-patch");

var documentA = {
    type: "Test document",
    arrayWillChange: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
    numberWillChange: 1,
    prop1: 1,
    prop2: {
        test1: 123,
        test2: 456
    },
    prop3: null
};

var documentB = {
    type: "Test document",
    arrayWillChange: [1, 2, 3, 4, 5],
    numberWillChange: 2,
    prop1: 1,
    prop2: {
        test1: 123,
        test2: 456
    },
    prop3: null
};

var diff = SmallJsonPatch.diff(documentA, documentB);

// diff = [
//   {
//     "op": "replace",
//     "path": "/numberWillChange",
//     "value": 2
//   },
//   {
//     "op": "replace",
//     "path": "/arrayWillChange",
//     "value": [1, 2, 3, 4, 5]
//   }
// ]

```

The diff can be used as input to a patch operation for any [RFC6902](http://tools.ietf.org/html/rfc6902) compliant library. For instance could the [fast-json-patch](https://www.npmjs.com/package/fast-json-patch) library be used like this:

```js
var FastJsonPatch = require("fast-json-patch");

var documentB = FastJsonPatch.applyPatch(documentA, diff);
```

## Typescript

This library is written in Typescript and the Typescript definitions are included.

## Complexity

The algorithm loops through the paths in the objects at most 2 times giving it a complexity of O(n) where n is the number of unique paths in the objects that are diffed.

## Main logic (simplified)

The paths in the old object and new object are traversed. During this traversal the information needed to create a small diff is gathered and stored in a tree structure. For each node the correct diff operation is determined. There are 4 possible operations: "replace", "remove", "children" and "none.

* "replace" = Perform replace operation for this node's path with the value at this path in the new object.

* "remove" = Perform remove operation for this node's path.

* "none" = Do nothing. The values match.

* "children" = Perform the operations dictated by this node's children

The length of the diff required to perform the operation is determined and stored as "diffSize". When the operation is "children" the diffSize is the sum of the children's diffSizes.

When determining what operation should be performed the diffSize of using "children" is compared to the diffSize of performing a "replace".

==> small-json-diff-1.1.1.tgz: package/README.md <==
# small-json-diff

Library for creating small [RFC6902](http://tools.ietf.org/html/rfc6902) compliant json diffs. The char length of the diff is guaranteed to be smaller than that of the diff produced by replacing the entire object. This is recursively true for each sub-object.

The algorithm will only produce "remove" and "replace" operations. The diff produced will be close to the smallest possible as long as no arrays are used. The reason is that it's hard to detect when items are moved (gets a new index position) inside an array. When that happens the item will be compared to the item that was at the new position before the move. But at least the diff size required to modify an array will never be larger than the diff size required to replace the entire array with the new array.

## Install

Install the current version (and save it as a dependency):

```sh
$ npm install small-json-patch --save
```

## Usage

```js
var SmallJsonDiff = require("small-json-diff");

var documentA = {
    type: "Test document",
    arrayWillChange: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
    numberWillChange: 1,
    prop1: 1,
    prop2: {
        test1: 123,
        test2: 456
    },
    prop3: null
};

var documentB = {
    type: "Test document",
    arrayWillChange: [1, 2, 3, 4, 5],
    numberWillChange: 2,
    prop1: 1,
    prop2: {
        test1: 123,
        test2: 456
    },
    prop3: null
};

var diff = SmallJsonDiff.diff(documentA, documentB);

// diff = [
//   {
//     "op": "replace",
//     "path": "/numberWillChange",
//     "value": 2
//   },
//   {
//     "op": "replace",
//     "path": "/arrayWillChange",
//     "value": [1, 2, 3, 4, 5]
//   }
// ]

```

The diff can be used as input to a patch operation for any [RFC6902](http://tools.ietf.org/html/rfc6902) compliant library. For instance could the [fast-json-patch](https://www.npmjs.com/package/fast-json-patch) library be used like this:

```js
var FastJsonPatch = require("fast-json-patch");

var documentB = FastJsonPatch.applyPatch(documentA, diff);
```

## Typescript

This library is written in Typescript and the Typescript definitions are included.

## Complexity

The algorithm loops through the paths in the objects at most 2 times giving it a complexity of O(n) where n is the number of unique paths in the objects that are diffed.

## Main logic (simplified)

The paths in the old object and new object are traversed. During this traversal the information needed to create a small diff is gathered and stored in a tree structure. For each node the correct diff operation is determined. There are 4 possible operations: "replace", "remove", "children" and "none.

* "replace" = Perform replace operation for this node's path with the value at this path in the new object.

* "remove" = Perform remove operation for this node's path.

* "none" = Do nothing. The values match.

* "children" = Perform the operations dictated by this node's children

The length of the diff required to perform the operation is determined and stored as "diffSize". When the operation is "children" the diffSize is the sum of the children's diffSizes.

When determining what operation should be performed the diffSize of using "children" is compared to the diffSize of performing a "replace".
